@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2003-2019 VMware, Inc. All rights reserved.
+ * Copyright (C) 2003-2023 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -81,6 +81,11 @@ extern "C" {
 #define ASOCKERR_NETUNREACH        14
 #define ASOCKERR_ADDRUNRESV        15
 #define ASOCKERR_BUSY              16
+#define ASOCKERR_PROXY_NEEDS_AUTHENTICATION  17
+#define ASOCKERR_PROXY_CONNECT_FAILED        18
+#define ASOCKERR_WEBSOCK_UPGRADE_NOT_FOUND   19
+#define ASOCKERR_WEBSOCK_TOO_MANY_CONNECTION 20
+#define ASOCKERR_PROXY_INVALID_OR_NOT_SUPPORTED 21
 
 /*
  * Cross-platform codes for AsyncSocket_GetGenericError():
@@ -95,6 +100,10 @@ extern "C" {
 #define ASOCK_ECONNRESET        WSAECONNRESET
 #define ASOCK_ECONNABORTED      WSAECONNABORTED
 #define ASOCK_EPIPE             ERROR_NO_DATA
+#define ASOCK_EHOSTUNREACH      WSAEHOSTUNREACH
+#define ASOCK_ETIMEDOUT         WSAETIMEDOUT
+#define ASOCK_ECONNREFUSED      WSAECONNREFUSED
+#define ASOCK_EACCES            WSAEACCES
 #else
 #define ASOCK_ENOTCONN          ENOTCONN
 #define ASOCK_ENOTSOCK          ENOTSOCK
@@ -105,6 +114,10 @@ extern "C" {
 #define ASOCK_ECONNRESET        ECONNRESET
 #define ASOCK_ECONNABORTED      ECONNABORTED
 #define ASOCK_EPIPE             EPIPE
+#define ASOCK_EHOSTUNREACH      EHOSTUNREACH
+#define ASOCK_ETIMEDOUT         ETIMEDOUT
+#define ASOCK_ECONNREFUSED      ECONNREFUSED
+#define ASOCK_EACCES            EACCES
 #endif
 
 /*
@@ -187,6 +200,7 @@ typedef enum AsyncSocketState {
    AsyncSocketConnected,
    AsyncSocketCBCancelled,
    AsyncSocketClosed,
+   AsyncSocketConnectedRdOnly,
 } AsyncSocketState;
 
 
@@ -383,6 +397,12 @@ int AsyncSocket_GetFd(AsyncSocket *asock);
 int AsyncSocket_GetRemoteIPStr(AsyncSocket *asock,
                                const char **ipStr);
 
+/*
+ * Return the remote port associated with this socket if applicable
+ */
+int AsyncSocket_GetRemotePort(AsyncSocket *asock,
+                              uint32 *port);
+
 int AsyncSocket_GetLocalVMCIAddress(AsyncSocket *asock,
                                     uint32 *cid, uint32 *port);
 int AsyncSocket_GetRemoteVMCIAddress(AsyncSocket *asock,
@@ -426,6 +446,10 @@ typedef int (*AsyncWebSocketHandleUpgradeRequestFn) (AsyncSocket *asock,
                                                      const char *httpRequest,
                                                      char **httpResponse);
 
+typedef int (*AsyncWebSocketHandoffSocketFn) (AsyncSocket *asock, void *cbData,
+                                              void *buf, uint32 bufLen,
+                                              uint32 currentPos);
+
 /*
  * Listen on port and fire callback with new asock
  */
@@ -455,16 +479,28 @@ AsyncSocket *AsyncSocket_ListenWebSocket(const char *addrStr,
                                          AsyncSocketPollParams *pollParams,
                                          void *sslCtx,
                                          int *outError);
-AsyncSocket *AsyncSocket_ListenWebSocketEx(const char *addrStr,
-                                           unsigned int port,
-                                           Bool useSSL,
-                                           const char *protocols[],
-                                           AsyncSocketConnectFn connectFn,
-                                           void *clientData,
-                                           AsyncSocketPollParams *pollParams,
-                                           void *sslCtx,
-                                           AsyncWebSocketHandleUpgradeRequestFn handleUpgradeRequestFn,
-                                           int *outError);
+AsyncSocket *AsyncSocket_PrepareListenWebSocket(Bool useSSL,
+                                                 const char *protocols[],
+                                                 AsyncSocketConnectFn connectFn,
+                                                 void *clientData,
+                                                 AsyncSocketPollParams *pollParams,
+                                                 void *sslCtx,
+                                                 AsyncWebSocketHandleUpgradeRequestFn handleUpgradeRequestFn,
+                                                 AsyncWebSocketHandoffSocketFn alpnCb,
+                                                 const char* alpn);
+AsyncSocket *AsyncSocket_RegisterListenWebSocket(AsyncSocket *asock,
+                                                 const char *addrStr,
+                                                 unsigned int port,
+                                                 AsyncSocketPollParams *pollParams,
+                                                 int *outError);
+AsyncSocket* AsyncSocket_UpgradeToWebSocket(AsyncSocket *asock,
+                                            const char *protocols[],
+                                            AsyncSocketConnectFn connectFn,
+                                            void *clientData,
+                                            Bool useSSL,
+                                            void *sslCtx,
+                                            AsyncWebSocketHandleUpgradeRequestFn handleUpgradeRequestFn,
+                                            int *outError);
 
 #ifndef _WIN32
 AsyncSocket *AsyncSocket_ListenWebSocketUDS(const char *pipeName,
@@ -483,7 +519,11 @@ AsyncSocket *AsyncSocket_ListenSocketUDS(const char *pipeName,
 
 
 /*
- * Connect to address:port and fire callback with new asock
+ * Connect to address:port and fire callback with new asock.
+ * If a custom error handler is needed, call AsyncSocket_SetErrorFn immediately
+ * after the new asock is created. If this is done on a thread that is not the
+ * poll thread, both calls should be done under the asyncsocket lock (passed
+ * via pollParams).
  */
 AsyncSocket *AsyncSocket_Connect(const char *hostname,
                                  unsigned int port,
@@ -539,6 +579,16 @@ AsyncSocket_CreateNamedPipe(const char *pipeName,
                             DWORD accessType,
                             AsyncSocketPollParams *pollParams,
                             int *error);
+AsyncSocket *
+AsyncSocket_AttachToNamedPipe(HANDLE handle, AsyncSocketPollParams *pollParams,
+                              int *outError);
+
+/*
+ * Obtain the client process id of the given named pipe
+ */
+Bool
+AsyncSocket_GetNamedPipeClientProcessId(AsyncSocket* asock,
+                                        PULONG clientPid);
 #endif
 
 AsyncSocket *
@@ -558,9 +608,11 @@ AsyncSocket_ConnectWebSocket(const char *url,
  */
 Bool AsyncSocket_ConnectSSL(AsyncSocket *asock,
                             struct _SSLVerifyParam *verifyParam,
+                            const char *hostname,
                             void *sslContext);
 int AsyncSocket_StartSslConnect(AsyncSocket *asock,
                                 struct _SSLVerifyParam *verifyParam,
+                                const char *hostname,
                                 void *sslCtx,
                                 AsyncSocketSslConnectFn sslConnectFn,
                                 void *clientData);
@@ -633,6 +685,11 @@ int AsyncSocket_Recv(AsyncSocket *asock, void *buf, int len, void *cb, void *cbD
 int AsyncSocket_RecvPartial(AsyncSocket *asock, void *buf, int len,
                             void *cb, void *cbData);
 
+int AsyncSocket_Peek(AsyncSocket *asock, void *buf, int len, void *cb, void *cbData);
+
+int AsyncSocket_PeekPartial(AsyncSocket *asock, void *buf, int len,
+                            void *cb, void *cbData);
+
 /*
  * Specify the amount of data to receive and the receive function to call.
  */
@@ -666,6 +723,8 @@ int AsyncSocket_SendBlocking(AsyncSocket *asock,
  */
 int AsyncSocket_Send(AsyncSocket *asock, void *buf, int len,
                       AsyncSocketSendFn sendFn, void *clientData);
+int AsyncSocket_SendWithFd(AsyncSocket *asock, void *buf, int len, int passFd,
+                           AsyncSocketSendFn sendFn, void *clientData);
 
 int AsyncSocket_IsSendBufferFull(AsyncSocket *asock);
 int AsyncSocket_GetNetworkStats(AsyncSocket *asock,
@@ -681,7 +740,9 @@ int AsyncSocket_CancelCbForClose(AsyncSocket *asock);
 
 /*
  * Set the error handler to invoke on I/O errors (default is to close the
- * socket)
+ * socket). This should be done immediately after an asyncsocket is created.
+ * If this is done on a thread that is not the poll thread, the asyncsocket
+ * lock (passed via pollParams) should be held throughout.
  */
 int AsyncSocket_SetErrorFn(AsyncSocket *asock, AsyncSocketErrorFn errorFn,
                            void *clientData);
@@ -693,6 +754,11 @@ int AsyncSocket_SetCloseOptions(AsyncSocket *asock,
                                 int flushEnabledMaxWaitMsec,
                                 AsyncSocketCloseFn closeCb);
 
+/*
+ * Close the write side of the connection.
+ */
+int AsyncSocket_CloseWrite(AsyncSocket *asock);
+
 /*
  * Close the connection and destroy the asock.
  */
@@ -711,10 +777,10 @@ char *AsyncSocket_GetWebSocketCookie(AsyncSocket *asock);
 /*
  * Set the Cookie  for a websocket connection
  */
-int AsyncSocket_SetWebSocketCookie(AsyncSocket *asock,         // IN
-                                   void *clientData,           // IN
-                                   const char *path,           // IN
-                                   const char *sessionId);     // IN
+int AsyncSocket_SetWebSocketCookie(AsyncSocket *asock,
+                                   void *clientData,
+                                   const char *path,
+                                   const char *sessionId);
 
 /*
  * Retrieve the close status, if received, for a websocket connection
@@ -726,6 +792,19 @@ uint16 AsyncSocket_GetWebSocketCloseStatus(AsyncSocket *asock);
  */
 const char *AsyncSocket_GetWebSocketProtocol(AsyncSocket *asock);
 
+/*
+ * Set the flag for whether or not to delay websocket upgrade response
+ */
+int AsyncSocket_SetDelayWebSocketUpgradeResponse(AsyncSocket *asock,
+                                                 Bool delayWebSocketUpgradeResponse);
+
+/*
+ * Send the websocket upgrade response
+ */
+void
+AsyncSocket_WebSocketServerSendUpgradeResponse(AsyncSocket *base,
+                                               char *httpResponseTemp);
+
 /*
  * Get error code for websocket failure
  */
@@ -738,7 +817,7 @@ const char * stristr(const char *s, const char *find);
  */
 Bool AsyncSocket_WebSocketParseURL(const char *url, char **hostname,
                                    unsigned int *port, Bool *useSSL,
-                                   char **relativeURL);
+                                   char **relativeURL, char **uriHostname);
 
 /*
  * Find and return the value for the given header key in the supplied buffer
@@ -746,33 +825,55 @@ Bool AsyncSocket_WebSocketParseURL(const char *url, char **hostname,
 char *AsyncSocket_WebSocketGetHttpHeader(const char *request,
                                          const char *webKey);
 
+unsigned AsyncSocket_WebSocketGetNumAccepted(AsyncSocket *asock);
+
+Bool AsyncSocket_SetKeepAlive(AsyncSocket *asock, int keepIdle);
+
+/*
+ * Send an HTTP error code, only valid on AsyncWebSockets
+ */
+void AsyncSocket_WebSocketServerSendError(AsyncSocket *asock, const char *text);
+
+
 /*
  * Some logging macros for convenience
  */
 #define ASOCKPREFIX "SOCKET "
 
-#define ASOCKWARN(_asock, _warnargs)                                 \
-   do {                                                              \
-      Warning(ASOCKPREFIX "%d (%d) ",                                \
-              AsyncSocket_GetID(_asock), AsyncSocket_GetFd(_asock)); \
-      Warning _warnargs;                                             \
-   } while (0)
+/* gcc needs special syntax to handle zero-length variadic arguments */
+#if defined(_MSC_VER)
+#define ASOCKWARN(_asock, fmt, ...)                               \
+   Warning(ASOCKPREFIX "%d (%d) " fmt, AsyncSocket_GetID(_asock), \
+           AsyncSocket_GetFd(_asock), __VA_ARGS__)
 
-#define ASOCKLG0(_asock, _logargs)                               \
-   do {                                                          \
-      Log(ASOCKPREFIX "%d (%d) ",                                \
-          AsyncSocket_GetID(_asock), AsyncSocket_GetFd(_asock)); \
-      Log _logargs;                                              \
-   } while (0)
+#define ASOCKLG0(_asock, fmt, ...)                            \
+   Log(ASOCKPREFIX "%d (%d) " fmt, AsyncSocket_GetID(_asock), \
+       AsyncSocket_GetFd(_asock), __VA_ARGS__)
 
-#define ASOCKLOG(_level, _asock, _logargs)                            \
-   do {                                                               \
-      if (((_level) == 0) || DOLOG_BYNAME(asyncsocket, (_level))) {   \
-         Log(ASOCKPREFIX "%d (%d) ",                                  \
-             AsyncSocket_GetID((_asock)), AsyncSocket_GetFd(_asock)); \
-         Log _logargs;                                                \
-      }                                                               \
-   } while(0)
+#define ASOCKLOG(_level, _asock, fmt, ...)                          \
+   do {                                                             \
+      if (((_level) == 0) || DOLOG_BYNAME(asyncsocket, (_level))) { \
+         Log(ASOCKPREFIX "%d (%d) " fmt, AsyncSocket_GetID(_asock), \
+             AsyncSocket_GetFd(_asock), __VA_ARGS__);               \
+      }                                                             \
+   } while (0)
+#else
+#define ASOCKWARN(_asock, fmt, ...)                               \
+   Warning(ASOCKPREFIX "%d (%d) " fmt, AsyncSocket_GetID(_asock), \
+           AsyncSocket_GetFd(_asock), ##__VA_ARGS__)
+
+#define ASOCKLG0(_asock, fmt, ...)                            \
+   Log(ASOCKPREFIX "%d (%d) " fmt, AsyncSocket_GetID(_asock), \
+       AsyncSocket_GetFd(_asock), ##__VA_ARGS__)
+
+#define ASOCKLOG(_level, _asock, fmt, ...)                          \
+   do {                                                             \
+      if (((_level) == 0) || DOLOG_BYNAME(asyncsocket, (_level))) { \
+         Log(ASOCKPREFIX "%d (%d) " fmt, AsyncSocket_GetID(_asock), \
+             AsyncSocket_GetFd(_asock), ##__VA_ARGS__);             \
+      }                                                             \
+   } while (0)
+#endif
 
 #if defined(__cplusplus)
 }  // extern "C"
